/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.bittorrent.client;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.ServerSocket;
import java.net.URL;
import java.net.URLConnection;
import java.util.Timer;
import java.util.TimerTask;

import org.klomp.snark.ConnectionAcceptor;
import org.klomp.snark.MetaInfo;
import org.klomp.snark.PeerAcceptor;
import org.klomp.snark.PeerCoordinator;
import org.klomp.snark.PeerMonitorTask;
import org.klomp.snark.Storage;
import org.klomp.snark.StorageListener;
import org.klomp.snark.TrackerClient;
import org.klomp.snark.bencode.BDecoder;
import org.klomp.snark.cmd.ConsoleStorageReporter;
import org.openmis.bittorrent.common.util.BittorrentUtils;

public class OpenMISClientBittorrent implements ClientBittorrent {

	private TrackerClient trackerclient = null;
	
    public final static int MIN_PORT = 6881;
    public final static int MAX_PORT = 6900;
    
    private int user_port = -1;
    private StorageListener slistener = null;
    private String destDownload = null;
    private String torrentURL;
    private Timer taskPeer = null;
    private TimerTask monitor = null;
    private ServerSocket serverSocket = null;
    
    private ConnectionAcceptor acceptor = null;
    private PeerAcceptor peeracceptor = null;
    private PeerCoordinator coordinator = null;
    private Storage storage = null;
    private MetaInfo info = null;
    
    private boolean monitorTask = false;
	
	public OpenMISClientBittorrent(int user_port, String torrentURL, String dirDestDownload, boolean monitorTask) throws IOException{
		this.user_port = user_port;
	
		this.destDownload = dirDestDownload;
		this.torrentURL = torrentURL;
		this.monitorTask = monitorTask;
		
		info = this.addTorrent(torrentURL);
	}
	
	public OpenMISClientBittorrent(String torrentURL, String dirDestDownload, boolean monitorTask) throws IOException{
		this(-1, torrentURL, dirDestDownload, monitorTask);
	}
	
	public OpenMISClientBittorrent(String torrentURL, String dirDestDownload) throws IOException{
		this(-1, torrentURL, dirDestDownload, false);
	}
	
	public void buildStorage(StorageListener slistener) throws IOException{
        // When the metainfo torrent was created from an existing file/dir
        // it already exists.
		this.slistener = slistener;
        storage = new Storage(info, slistener);
        storage.check(destDownload);  
	}
	
	public int getUserPort(){
		return this.user_port;
	}
	
	public StorageListener getStorageListener(){
		return slistener;
	}
	
	public String getTorrentURL() {
		return torrentURL;
	}
	
	public TrackerClient getTrackerclient() {
		return trackerclient;
	}
	
	public ServerSocket getServerSocket(){
		return serverSocket;
	}
	
	public ConnectionAcceptor getAcceptor() {
		return acceptor;
	}
	
	public PeerCoordinator getCoordinator() {
		return coordinator;
	}
	
	public String getDirDestDownload() {
		return destDownload;
	}
	
	public MetaInfo getMetaInfo() {
		return info;
	}
	
	public PeerAcceptor getPeeracceptor() {
		return peeracceptor;
	}
	
	public Storage getStorage() {
		return storage;
	}
	
	public boolean isCompleteDownload(){
		return storage.complete();
	}
	
	public boolean isMonitoredTask() {
		return monitorTask;
	}
	
	public void setMonitorTask(boolean monitorTask) {
		this.monitorTask = monitorTask;
		if(monitorTask){
			if(taskPeer == null && monitor == null){	
		        taskPeer = new Timer(true);
		        monitor = new OpenMISPeerMonitorTask(coordinator, this);
		        taskPeer.schedule(monitor, PeerMonitorTask.MONITOR_PERIOD, PeerMonitorTask.MONITOR_PERIOD);
			}
		}else{
			if(taskPeer != null && monitor != null){
				taskPeer.cancel();
				monitor.cancel();
				
				taskPeer = null;
				monitor = null;
				
			}
		}
	}
	
	public void stopClientTracker()throws IOException{

		setMonitorTask(false);
		if(storage != null)
			storage.close();
		if(coordinator != null)
			coordinator.halt();
		if(trackerclient != null)
			trackerclient.halt();
		if(acceptor != null)
			acceptor.halt();
		if(slistener != null)
			slistener = null;
		
		if(serverSocket != null){
			try{
				System.out.println("CLOSING SOCKET  : "+serverSocket);
				serverSocket.close();	//to be sure terminate the socket
			}catch(Exception ex){
				System.out.println("FAILED CLOSING SOCKET : "+ serverSocket);
			}
		}
	}
	
	public void startClientTracker() throws IOException{
        
		if(storage == null)
			buildStorage(new OpenMISStorageListener(this));
		
        /** The coordinator managing our peers */
        coordinator = new PeerCoordinator(
        		BittorrentUtils.generateNewSnarkPeerID()
        		, info
        		, storage
        		, null);
        
        peeracceptor = new PeerAcceptor(coordinator);
        serverSocket = getServerSocket(getUserPort());
        //System.out.println("SERVER SOCKET ATTRIBUTE : "+serverSocket);
        
        
        acceptor = new ConnectionAcceptor(
        		 serverSocket
        		, null
        		, peeracceptor);
        acceptor.start();
        
        /** Obtains information on new peers. */
        trackerclient = new TrackerClient(info, coordinator, acceptor.getPort());
        trackerclient.start();
        coordinator.setTracker(trackerclient);
        
        setMonitorTask(isMonitoredTask());
        
        //----- Control if the file is already completed and so stop te client ---- 
        if(storage.complete())
        	stopClientTracker();

	}
	
	private ServerSocket getServerSocket(int user_port)throws IOException{
		ServerSocket serversocket = null;
		IOException lastException = null;
		int port = -1;
		
        if (user_port != -1) {	
            port = user_port;
            try {
                serversocket = new ServerSocket(port);
            } catch (IOException ioe) {
                lastException = ioe;
            }
        } else {
            for (port = MIN_PORT; serversocket == null && port <= MAX_PORT; port++) {
                try {
                    serversocket = new ServerSocket(port);
                } catch (IOException ioe) {
                    lastException = ioe;
                }
            }
        }
        if (serversocket == null) {
            String message = "Cannot accept incoming connections ";
            if (user_port == -1) {
                message = message + "tried ports " + MIN_PORT + " - "+ MAX_PORT;
            } else {
                message = message + "on port " + user_port;
            }
            port = -1;
            throw new IOException(message);
        }
        
        return serversocket;
	}
	
	
	private MetaInfo addTorrent(String torrent) throws IOException{
        // Figure out what the torrent argument represents.
        File f = null;
        InputStream in;
        f = new File(torrent);
        if (f.exists()) {
            in = new FileInputStream(f);
        } else {
            URL u = new URL(torrent);
            URLConnection c = u.openConnection();
            c.connect();
            in = c.getInputStream();

            if (c instanceof HttpURLConnection) {
                // Check whether the page exists
                int code = ((HttpURLConnection)c).getResponseCode();
                if (code / 100 != 2) {
                    // responses
                    throw new IOException("Loading page '" + torrent + "' gave error code " + code + ", it probably doesn't exists");
                }
            }
        }
        return new MetaInfo(new BDecoder(in));
	}
}
