/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Tellus module - for mobile GIS applications
 * Copyright (C) 2004-2006 Philippe Delrieu - 2008-2009 Manuel Gomez
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org; mgomez@ubikis.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author OpenMosbileIS: Philippe Delrieu
 *  Author Tellus module : Manuel Gomez
 */
package org.openmis.bittorrent.client;

import java.io.IOException;
import java.util.Iterator;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.klomp.snark.Peer;
import org.klomp.snark.PeerCoordinator;

/**
 * TimerTask that monitors the peers and total up/download speeds. Works
 * together with the main Snark class to report periodical statistics.
 */
public class OpenMISPeerMonitorTask extends TimerTask {
	
    /** The Java logger used to process our log events. */
    protected static final Logger log = Logger.getLogger("org.openmis.bittorrent.client");

	public static final long MONITOR_PERIOD = 10 * 1000; // Ten seconds.

    private static final long KILOPERSECOND = 1024 * (MONITOR_PERIOD / 1000);

    private final PeerCoordinator coordinator;

    private long lastDownloaded = 0;

    private long lastUploaded = 0;
    
    private ClientBittorrent client = null;

    public OpenMISPeerMonitorTask (PeerCoordinator coordinator, ClientBittorrent client)
    {
        this.coordinator = coordinator;
        this.client = client;
    }


    public void run ()
    {
        // Get some statistics
        int peers = 0;
        int uploaders = 0;
        int downloaders = 0;
        int interested = 0;
        int interesting = 0;
        int choking = 0;
        int choked = 0;

        synchronized (coordinator.peers) {
            Iterator it = coordinator.peers.iterator();
            while (it.hasNext()) {
                Peer peer = (Peer)it.next();

                // Don't list dying peers
                if (!peer.isConnected()) {
                    continue;
                }

                peers++;

                if (!peer.isChoking()) {
                    uploaders++;
                }
                if (!peer.isChoked() && peer.isInteresting()) {
                    downloaders++;
                }
                if (peer.isInterested()) {
                    interested++;
                }
                if (peer.isInteresting()) {
                    interesting++;
                }
                if (peer.isChoking()) {
                    choking++;
                }
                if (peer.isChoked()) {
                    choked++;
                }
            }
        }

        // Print some statistics
        long downloaded = coordinator.getDownloaded();
        String totalDown;
        if (downloaded >= 10 * 1024 * 1024) {
            totalDown = (downloaded / (1024 * 1024)) + "MB";
        } else {
            totalDown = (downloaded / 1024) + "KB";
        }
        long uploaded = coordinator.getUploaded();
        String totalUp;
        if (uploaded >= 10 * 1024 * 1024) {
            totalUp = (uploaded / (1024 * 1024)) + "MB";
        } else {
            totalUp = (uploaded / 1024) + "KB";
        }

        int needP = coordinator.storage.needed();
        long needMB = needP * coordinator.metainfo.getPieceLength(0)
            / (1024 * 1024);
        int totalP = coordinator.metainfo.getPieces();
        long totalMB = coordinator.metainfo.getTotalLength() / (1024 * 1024);

        /*log.log(Level.INFO, "Down: " + (downloaded - lastDownloaded)
            / KILOPERSECOND + "KB/s" + " (" + totalDown + ")" + " Up: "
            + (uploaded - lastUploaded) / KILOPERSECOND + "KB/s" + " ("
            + totalUp + ")" + " Need " + needP + " (" + needMB + "MB)" + " of "
            + totalP + " (" + totalMB + "MB)" + " pieces");
        log.log(Level.INFO, peers + ": Download #" + downloaders + " Upload #"
            + uploaders + " Interested #" + interested + " Interesting #"
            + interesting + " Choking #" + choking + " Choked #" + choked);*/
        
        System.out.println();
        System.out.println();
        
        System.out.println("Name: " + coordinator.metainfo.getName());
        	System.out.println("Total peers: "
                + coordinator.getPeers());
            System.out.println("Total size: "
                + coordinator.metainfo.getTotalLength() / (1024 * 1024) + " MB");
            System.out.println("Total remaining: "
                + coordinator.getLeft() / (1024 * 1024) + " MB");
            System.out.println("Total downloaded: "
                + coordinator.getDownloaded() / (1024 * 1024) + " MB");
            System.out.println("Total uploaded: "
                + coordinator.getUploaded() / (1024 * 1024) + " MB");
            System.out.println("Down: " + (downloaded - lastDownloaded)
            / KILOPERSECOND + "KB/s" + " (" + totalDown + ")" + " Up: "
            + (uploaded - lastUploaded) / KILOPERSECOND + "KB/s" + " ("
            + totalUp + ")" + " Need " + needP + " (" + needMB + "MB)" + " of "
            + totalP + " (" + totalMB + "MB)" + " pieces");
            
            if(coordinator.completed()){
            	 System.out.println("DOWNLOAD TERMINATE FOR : " + coordinator.metainfo.getName());
            }
        

        
        
        lastDownloaded = downloaded;
        lastUploaded = uploaded;
    }



}
