/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Panel;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.Refreshable;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerGenericRaster;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.viewport.ExtentEvent;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPortListener;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.icons.IconResourceReader;

/**
 * This class represents the TOC (the list of layers in the curretn view/project)
 * Keeps a reference to the MapControl and consists of a set of rows
 * (class LayersTableItem), one for each layer.
 * 
 * Changes in the order of the layers are performed by this class, and it
 * calls the MapContext to perform the redrawing.
 * 
 * On the right side of the TOC there are some buttons that act on the selected
 * layer. The number of selected layers can be zero or one.
 * 
 * The forst row of the table (the one with the names of the columns) is also a
 * LayersTableItem, created with a different constructor. 
 * 
 * @see es.prodevelop.gvsig.mobile.gui.canvas.MapControl
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.ViewLayersGraphicStatus
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.LayersTableItem
 * 
 * @author jldominguez
 *
 */
public class LayersTable extends Panel implements Refreshable, ViewPortListener {
	
	private static Logger logger = Logger.getLogger(LayersTable.class);
	private ArrayList list = new ArrayList();
	private int width = 0;
	private ViewLayersGraphicStatus layersGraphicStatus = null;
	private boolean processing = false;
	private MapControl mapControl;

	public static final int REMOVED_OK = 1;
	public static final int REMOVED_FAIL = 0;


	/**
	 * Constructor. 
	 * 
	 * @param w width in pixels of the table.
	 * @param mc A reference to the MapControl
	 */
	public LayersTable(int w, MapControl mc) {
		width = w;
		mapControl = mc;
		setLayout(null);
		setSize(width, LayersTableItem.ROW_STEP);
		setBackground(Color.WHITE);
		
		LayersTableItem newitem = new LayersTableItem(w, list.size(), this);
		newitem.setPosition(0, w);
		list.add(newitem);
		add(newitem);
		
	}
	
	/**
	 * Gets the current scale. The scale influences the TOC because there is an icon
	 * to show whether the current scale is within each layer's scale limits.
	 *  
	 * @return the current scale (returns x when the scale is 1:x)
	 */
	public double getScale() {
		return mapControl.getViewPort().getScale();
	}
	
	/**
	 * Removes i-th layer from the TOC
	 * 
	 * @param one_based_ind the index of the layer to be removed
	 */
	public int removeItem(int one_based_ind) {

		if ((one_based_ind > (list.size() - 1))
		|| (one_based_ind == 0)) {
			return -1;
		}

		LayersTableItem remove_item = (LayersTableItem) list.get(one_based_ind);
		FLayer remove_lyr = remove_item.getLayer();

		if(!remove_lyr.isEditing()){

			LayersTableItem item_table_item;
			int initial_size = list.size();

			for (int i=1; i<initial_size; i++) {
				item_table_item = (LayersTableItem) list.get(i);
				if (item_table_item.getLayer() == remove_lyr) {

					list.remove(item_table_item);
					remove(item_table_item);

					for (int reset=i; reset<(initial_size-1); reset++) {
						item_table_item = (LayersTableItem) list.get(reset);
						item_table_item.updateBounds(reset);
					}
					
					mapControl.removeLayer(remove_lyr);
					repaint();
					return 1;
				}
			}
			return 0;

		}else{

			return 0;

		}
		
	}
	
	/**
	 * Unselects all layers. 
	 *
	 */
	public void unselectAll() {
	
		int size = list.size();
		LayersTableItem item_table_item;
		
		for (int i=1; i<size; i++) {
			item_table_item = (LayersTableItem) list.get(i);
			item_table_item.setSelected(false);
		}
	}
	
	/**
	 * Adfds a new layer to the TOC.
	 * 
	 * @param lyr the layer to be added
	 * @param title whether it's the false layer that works as the title. must be false
	 * when used normally
	 * 
	 * @return the new layer count
	 */
	public int addItem(FLayer lyr, boolean title) {
		
		if (title) {
			// title table item
			setSize(getWidth(), LayersTableItem.ROW_STEP);
			LayersTableItem newitem = new LayersTableItem(lyr, width, this);
			newitem.setPosition(0, getWidth());
			list.add(newitem);
			add(newitem);
			processing = false;
			return 1;
			
		} else {
			
			int ind = list.size();
			setSize(getWidth(), (ind  + 1) * LayersTableItem.ROW_STEP);

			LayersTableItem newitem = new LayersTableItem(lyr, width, this);
			list.add(1, newitem);
			add(newitem, 1);
			
			for (int i=1; i<(ind + 1); i++) {
				((LayersTableItem) list.get(i)).setPosition(i, getWidth());
			}
			processing = false;
			return ind + 1;
		}
		
	}
	
	/**
	 * Get current layers. Iterates the list of LayersTableItem, exclkuding the first one
	 * (title)
	 * 
	 * @return array with current layers
	 */
	public FLayer[] getLayers() {
		
		int size = list.size() - 1; // first is title
		FLayer[] resp = new FLayer[size];
		for (int i=0; i<size; i++) {
			resp[i] = ((LayersTableItem) list.get(i + 1)).getLayer();
		}
		return resp;
	}

	/**
	 * Keeps a copy of the current graphic settings of the layers. It is iused to check
	 * whether a repainting is needed after the user has accessed the symbology
	 * dialog of any layer.
	 * 
	 *
	 */
	public void storeStatus() {
		layersGraphicStatus = new ViewLayersGraphicStatus(this);
	}

	/**
	 * Checks whether the symbology settings of the layers have changed. That would trigger
	 * a redrawing of the view.
	 * 
	 * @return whether the symbology settings of the layers have changed
	 */
	public boolean sameAsLastStore() {
		ViewLayersGraphicStatus current = new ViewLayersGraphicStatus(this);
		return layersGraphicStatus.isSame(current);
	}

	/**
	 * Unused. Needed to implement interface Refreshable.
	 */
	public void refresh(boolean update) {
	}

	/**
	 * Unused. Needed to implement interface Refreshable.
	 */
	public void redraw(boolean clear) {
	}

	/**
	 * Unused. Needed to implement interface Refreshable.
	 */
	public void setOffsetAndRefresh(int x, int y, boolean refresh) {
	}

	/**
	 * Unused. Needed to implement interface Refreshable.
	 */
	public void setProcessing(boolean b) {
		processing = b;
	}

	/**
	 * Unused. Needed to implement interface Refreshable.
	 */
	public boolean isProcessing() {
		return processing;
	}
	
	public void paint(Graphics g) {
		
		super.paint(g);
		if (processing) {
			int x = (getWidth() - GvsigMobileGUIConstants.PROCESS_ICON_SIZE) / 2;
			int y = (getHeight() - GvsigMobileGUIConstants.PROCESS_ICON_SIZE) / 2;
			g.drawImage(
					IconResourceReader.PROCESSING_ICON,
					x, y,
					null);
		}
		
	}
	
	/**
	 * Update scale visibility icons. This is called when the extent changes.
	 */
	public void updateScaleVisibilityIcons() {

		LayersTableItem item;
		FLayer lyr;
		ScaleVisibilityIcon svi;

		for (int i = 1; i < list.size(); i++) { // first item in list is the title
			item = (LayersTableItem) list.get(i);
			lyr = item.getLayer();
			svi = item.getScaleVisiLabel();
			
			if ((lyr.getMinScale() == -1) && (lyr.getMaxScale() == -1)) {
				
				svi.setState(ScaleVisibilityIcon.STATE_NOT_DEFINED);
				
			} else {

				if (lyr.isWithinScale(getScale())) {
					svi.setState(ScaleVisibilityIcon.STATE_VISIBLE);
				} else {
					svi.setState(ScaleVisibilityIcon.STATE_NOT_VISIBLE);
				}
				
			}
		}
	}

	/**
	 * Updates row associated with a layer
	 * @param lyr layer whose row must be updated,
	 */
	public void updateItem(FLayer lyr) {
		
		LayersTableItem item;
		for (int i=0; i<list.size(); i++) {
			item = (LayersTableItem) list.get(i);
			if (item.getLayer() == lyr) {
				item.getNameLabel().setText(lyr.getName());
				item.getVisibleCheck().setState(lyr.isVisible());
				//item.getInfoableCheck().setState(lyr.isInfoable());
				
				ScaleVisibilityIcon svi = item.getScaleVisiLabel();
				
				if ((lyr.getMinScale() == -1) && (lyr.getMaxScale() == -1)) {
					svi.setState(ScaleVisibilityIcon.STATE_NOT_DEFINED);
				} else {
					if (lyr.isWithinScale(getScale())) {
						svi.setState(ScaleVisibilityIcon.STATE_VISIBLE);
					} else {
						svi.setState(ScaleVisibilityIcon.STATE_NOT_VISIBLE);
					}
				}
			}
		}
	}

	public void moveSelectedDown() {
		int size = list.size();
		int index = getSelectedIndex();
		if (index == -1) return;
		if (index == (size - 1)) return; // last
		
		LayersTableItem a = (LayersTableItem) list.get(index);
		LayersTableItem b = (LayersTableItem) list.get(index + 1);
		a.updateBounds(index + 1);
		b.updateBounds(index);
		
		Utils.swap(list, index, index + 1);
		mapControl.swap(index - 1, index, false);
	}

	/**
	 * Moves up one position the selected layer.
	 *
	 */
	public void moveSelectedUp() {
		int index = getSelectedIndex();
		if (index == -1) return;
		if (index == 1) return; // first
		
		LayersTableItem a = (LayersTableItem) list.get(index);
		LayersTableItem b = (LayersTableItem) list.get(index - 1);
		a.updateBounds(index - 1);
		b.updateBounds(index);

		Utils.swap(list, index, index - 1);
		mapControl.swap(index - 1, index - 2, false);
	}
	
	public MapControl getMapControl() {
		return mapControl;
	}

	/**
	 * Gets the index of the elected layer.
	 * 
	 * @return the index of the elected layer
	 */
	public int getSelectedIndex() {
		int size = list.size();
		LayersTableItem item;
		for (int i=1; i<size; i++) {
			item = (LayersTableItem) list.get(i);
			if (item.isSelected()) return i;
		}
		return -1;
	}

	public int getWidth() {
		return width;
	}

	/**
	 * Performs a zoom to the extent of the selected layer.
	 *
	 */
	public void zoomToSelected() {
		
		int index = getSelectedIndex();
		if (index == -1) {
			logger.debug("Zoom to selected layer: no layer selected, nothing done.");
			return;
		}

		LayersTableItem item = (LayersTableItem) list.get(index);
		FLayer lyr = item.getLayer();

		Rectangle2D ext;
		try {
			ext = lyr.getFullExtent();
		} catch (FMapDriverException e) {
			logger.error("Zoom to selected layer, while gettinmg selected layer extent: " + e.getMessage());
			return;
		}
		
		if (ext == null) {
			logger.debug("Zoom to selected layer: selected layer returned NULL extent, nothing done.");
			return;
		}
		
		mapControl.getViewPort().setExtent(ext, true);
	}

	/**
	 * The TOC also listens to extent changes in the viewport, since the scale
	 * visibility icons need to be updated.
	 */
	public void extentChanged(ExtentEvent e, boolean repaint) {
		updateScaleVisibilityIcons();
	}
	
	public boolean isSomeoneEditing(int index){
		int size = list.size();
		LayersTableItem item;
		for (int i=1; i<size; i++) {
			item = (LayersTableItem) list.get(i);
			//if (item.isEditing()&& !(item.getIndex()==index)) return true;
		}
		return false;
	}
	
	public void dissableEditionCheckBoxes(int index){
		int size = list.size();
		LayersTableItem item;
		for (int i=1; i<size; i++) {
			item = (LayersTableItem) list.get(i);
			//if(index != item.getIndex()) item.setEnableDissabledEditionCheck(false);
		}
	}
	
	public void enableEditionCheckBoxes(){
		int size = list.size();
		LayersTableItem item;
		for (int i=1; i<size; i++) {
			item = (LayersTableItem) list.get(i);
			//item.setEnableDissabledEditionCheck(true);
		}
	}
	
	
}
